# frozen_string_literal: true

#
# Cookbook:: teleport-ce-cookbook
# Recipe:: server
#
# Copyright: 2019, Pharmony SA, All Rights Reserved.
#

require 'digest'

# `wait_command_output`
Chef::Resource::RubyBlock.send(:include, TeleportCookbook::StreamCommand)
# `build_dynamic_labels_for_node`
# `build_static_labels_for_node`
Chef::Resource::Template.send(:include, TeleportCookbook::Helper)

require 'securerandom'

include_recipe 'teleport-ce::common'

package 'curl' do
  action :install
end

# Installs the systemd unit for Teleport and enables it
systemd_unit 'teleport.service' do
  action %i(create enable)

  content <<-EOU.gsub(/^\s+/, '')
    [Unit]
    Description=Teleport SSH Service
    After=network.target

    [Service]
    Type=simple
    Restart=on-failure
    ExecStart=/usr/local/bin/teleport start --roles=auth,node,proxy --config=/etc/teleport.yaml --pid-file=/var/run/teleport.pid
    ExecReload=/bin/kill -HUP $MAINPID
    PIDFile=/var/run/teleport.pid

    [Install]
    WantedBy=multi-user.target
  EOU
end

# Generates and store a static token that will be used in order to allow nodes
# to join the Teleport server.
ruby_block 'generate and store a static token' do
  action  :run
  block   { node.normal['teleport']['static_token'] = SecureRandom.hex(16) }
  only_if { node['teleport']['static_token'].nil? }
end

# Generates the Teleport configuration file
template '/etc/teleport.yaml' do
  action :create
  source 'server.teleport.yaml.erb'
  owner 'root'
  group 'root'
  mode '0700'
  variables(
    dynamic_labels: build_dynamic_labels_for_node,
    static_labels: build_static_labels_for_node,
    static_token: lazy { node['teleport']['static_token'] }
  )
end

# Finally starts the teleport service
service 'teleport' do
  action [:restart]
end

ruby_block 'wait for the teleport to be up and running' do
  action :run
  block do
    timeout = 30
    Chef::Log.info 'Waiting for Teleport to boot ' \
                   "(timeout after #{timeout} seconds) ..."

    begin
      # Runs the given command, and look at its ouput if it matches the expected
      # regex until the end of the timeout.
      # Each time the output changes, the block is executed with its output.
      wait_command_output(
        'curl --insecure  https://localhost:3080/webapi/ping',
        /\"server_version\":\"/,
        timeout: timeout
      ) do |output|
        Chef::Log.info "Current Teleport webapi response: #{output.inspect}."
      end
    rescue TeleportCookbook::WaitCommandTimeoutError
      # When the timeout is reached, the WaitCommandTimeoutError is thrown
      Chef::Log.error 'Waiting for Teleport to be Running failed after ' \
                      "#{timeout} seconds."
      raise
    end
  end
end

# Retrieves the CA pin that will be used by nodes to join the
# Teleport server.
ruby_block 'retrieve and store a CA pin' do
  action :run
  block do
    ca_pin = shell_out(
      "tctl status | grep \"CA pin\" | awk '{print $3}'"
    ).stdout.gsub(/\n/, '')
    Chef::Log.info "Retrieved Teleport CA pin: #{ca_pin}"
    node.normal['teleport']['ca_pin'] = ca_pin
  end
end
